<?
/****************************************************************************
S7_FUNCTION
 + ist eine Funktionssammlung fr den Zugriff auf S7-Instanzen

Release: 0.9
ReleaseDate: 14.05.2014

ReleaseNote:
30.04.2009 - R0.3 - Gusenbauer
 + Function S7_VAR_IDs_READ geandert

02.06.2012 - R0.4 - Gusenbauer
 + Function S7_WRITE eingefuehrt

10.06.2012 - R0.5 - Gusenbauer
 + Check der Instanzen ob OKAY, ansonsten Abbruch
	- S7_VAR_IDs_READ
	- S7_WRITE
 + Auslesen der Verbindungsinstanz hat sich mit der IPS2.6 geaendert !!
   ==> daher wird ab 2.6 die "ConnectionID" verwendet
   ==> bis 2.5 aber die "ParentID"

23.12.2012 - R0.6 - Gusenbauer
 + Anpassungen an IPS2.7
   ==> alles auf IPS_SetProperty / IPS_GetProperty umgestellt
   
01.09.2013 - R0.7 - Gusenbauer
 + Anpassungen IPS3.0
	==> S7-Instanzenflag "ReadOnly" und "EmulateStatus" gesetzt
 + alten Kompatiblittscode entfernt ... daher erst ab 2.6 lauffaehig
 
13.05.2014 - R0.8 - Gusenbauer 
 + Neue Funktion zum anlegen von S7 - Instanzen (#101)

14.05.2014 - R0.9 - Gusenbauer
 + Function um Actionscript und Userprofil erweitert (#102)
 
*****************************************************************************/

include_once ('./common_function.php');


/****************************************************************************
S7_VAR_IDs_READ,
  liest das uebergebene Variablenarray "S7_VAR_IDs" von der "S7_Instanz_ID"
  und legt die Variablen entsprechend dem "S7_Path" ab

Example:    //index0   = VARIABLENBEZEICHNUNG in IPS
            //index1.0 = TYP          (0=bit, 1=byte, 2=word, 3=dword,
				//                         4=ShortInt, 5=SmallInt, 6=Integer, 7=Real)
            //index1.1 = AREA         (0=SystemInfo, 1=SystemFlags,
				//                         2=AnalogInput CPU200, 3=AnalogOutput CPU200,
				//                         4=Input, 5=Output, 6=Flags, 7=Datablock,
				//                         8=InstanzDaten, 9=LokaleDaten, 10=UnknownedArea
				//                         11=Zaehler, 12=Timer, 13=PEW/PAW)
            //index1.2 = AREAADDRESS  (Baustein)
            //index1.3 = ADDRES       (StartByte)
            //index1.4 = BIT          (Bit = wird nur bei TYP=0 verarbeitet)
            //index1.5 = CONVERT-S7-Value vor schreiben auf die IPS-Variable
            //                        (0=unverndert, 1=Dec2Hex, 2=Hex2Dec)

            $S7_VAR_IDs = array("M0.1"   => array(0,6,0,0,1,0),
				                    "E41.1"  => array(0,4,0,41,1,0));
            S7_VAR_IDs_READ(123456,"S7/TEST",$S7_VAR_IDs);

ReleaseNotes:
02.04.2009 Gusenbauer
 + Function erstellt und geprft
07.04.2009 Gusenbauer
 + Wenn Lesefehler oder Zuweisungsfehler wird ber ERR_COUNT gezaehlt
	und zurueckgegeben!!
18.04.2009 Gusenbauer
 + Pruefung eingefuehrt welche bei M0.1 die Leseroutine abbricht
   wenn 0 gelesen wird (am sinnvollsten als erste Variable uebergeben)
30.04.2009 Gusenbauer
 + Baustein um REAL erweitert und wird als FLOAT abgelegt, Anfrage von MICHELH
10.06.2012 Gusenbauer
 + Check der Instanzen ob OKAY,
	sonst Abbruch und RET >= 1000
	RET: 9999 = Instanzefehler
		  1000 = S7-Instanzenfehler
		  1001 = S7-Verbindungsinstanzenfehler
23.12.2012 Gusenbauer
 + Anpassungen IPS2.7
01.09.2013 Gusenbauer
 + Anpassungen IPS3.0
   ==> Instanzenflag "ReadOnly" und "EmulateStatus" gesetzt
 + alten Kompatiblittscode entfernt
 
*****************************************************************************/
function S7_VAR_IDs_READ($S7_Instanz_ID,$S7_Path,$S7_VAR_IDs)
{
  CheckCreateCategory($S7_Path);
  $err_count = 0;

  while($S7_VAR_ID = each($S7_VAR_IDs))
    {
		   //Wenn Variable noch nicht existiert
			if ($S7_VAR_ID[1][0] == 0)
				{
             $VarID = CheckCreateVariable($S7_Path . "/" . $S7_VAR_ID[0],0);  //boolean
				}
			else if ($S7_VAR_ID[1][0] == 7)
				{
             $VarID = CheckCreateVariable($S7_Path . "/" . $S7_VAR_ID[0],2);  //float
				}
			else
				{
             $VarID = CheckCreateVariable($S7_Path . "/" . $S7_VAR_ID[0],1);  //integer
				}

         IPS_SetProperty($S7_Instanz_ID,"DataType",$S7_VAR_ID[1][0]);
         IPS_SetProperty($S7_Instanz_ID,"Area",$S7_VAR_ID[1][1]);
         IPS_SetProperty($S7_Instanz_ID,"AreaAddress",$S7_VAR_ID[1][2]);
         IPS_SetProperty($S7_Instanz_ID,"Address",$S7_VAR_ID[1][3]);
         IPS_SetProperty($S7_Instanz_ID,"Bit",$S7_VAR_ID[1][4]);

			//Polltimer sicherheitshalber ausschalten
         IPS_SetProperty($S7_Instanz_ID,"Poller",0);
         
         //ReadOnly an der Instanz entfernen
         IPS_SetProperty($S7_Instanz_ID,"ReadOnly",false);
         
         //EmulateStatus an der Instanz entfernen
         IPS_SetProperty($S7_Instanz_ID,"EmulateStatus",false);

			//Instanzkonfiguration speichern
	      if (IPS_ApplyChanges($S7_Instanz_ID) == false)
	         {
             IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Instanzenaenderung konnte nicht gespeichert werden !! --> ABBRUCH !!");

			    echo ("ERROR: S7-Instanzenaenderung konnte nicht gespeichert werden !! ==> S7-Variable: " . $S7_VAR_ID[0] . "\r\n");
             $err_count = 9999;
			    return $err_count;
				}

	      //Check ob S7-Instanz okay ???
	      $instance = IPS_GetInstance($S7_Instanz_ID);
         if ($instance['InstanceStatus'] >= 104)
            {
             IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Instanzenfehler aufgetreten !! --> ABBRUCH !!");

			    echo ("ERROR: S7-Instanzenfehler aufgetreten !! ==> S7-Variable: " . $S7_VAR_ID[0] . "\r\n");
             $err_count = 1000;
			    return $err_count;
            }

	      //Check ob Verbindungsinstanz zur Steuerung wirklich steht ???
	      $instance = IPS_GetInstance($instance['ConnectionID']);
         if ($instance['InstanceStatus'] >= 104)
            {
             IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Verbindungsinstanzenfehler aufgetreten !! --> ABBRUCH !!");

 			    echo ("ERROR: S7-Verbindungsinstanzenfehler aufgetreten !! ==> S7-Variable: " . $S7_VAR_ID[0] . "\r\n");
             $err_count = 1001;
			    return $err_count;
            }

			if (S7_RequestRead($S7_Instanz_ID) == true)
				{
				 //Anpassung ==> IPS2.7
			    //$VarStatusID = IPS_GetStatusVariableID($S7_Instanz_ID,"Value");
               $VarStatusID = IPS_GetObjectIDByIdent('Value',$S7_Instanz_ID);
               
				 //Check ob eine konvertierung stattfinden soll
				 if ($S7_VAR_ID[1][5] == 1)
					 {
                 $S7_READ_VALUE = intval(hexdec(GetValue($VarStatusID)));
					 }
				 else if ($S7_VAR_ID[1][5] == 2)
					 {
                 $S7_READ_VALUE = intval(dechex(GetValue($VarStatusID)));
					 }
				 else
					 {
                 $S7_READ_VALUE = GetValue($VarStatusID);
					 }

				//Pruefung ob Variable M0.1 entspricht und wenn hier der Wert 0, dann lesen abbrechen
				//nicht so schoen aber Wirkungsvoll, denn beim Anlauf immer wieder 0 fuer alle Variablen gelesen

				if ($S7_VAR_ID[0] == "M0.1")
					{
					if($S7_READ_VALUE == false)
					  {
                  $err_count++;
  					   echo "FEHLER: M0.1 ist mit 0 gelesen worden --> Abbruch der Leseroutine\r\n";
						return $err_count;
					  }
					}

				//----------------------------------------

			   //S7_VAR_ID - Wert aktualisieren
			   if ($S7_VAR_ID[1][0] == 0)
				   {
				    if (SetValueBoolean($VarID,$S7_READ_VALUE) == false)
						 {
					     echo "FEHLER: Wert '" . $S7_READ_VALUE . "' konnte mit SetValueBoolean auf die ID: " . $VarID . " zuwegwiesen werden (" . $S7_Path . "/" . $S7_VAR_ID[0] . ")\r\n";
						 }
  				   }
			   else if ($S7_VAR_ID[1][0] == 7)  //REAL
			      {
				    if (SetValueFloat($VarID,$S7_READ_VALUE) == false)
						 {
					     echo "FEHLER: Wert '" . $S7_READ_VALUE . "' konnte mit SetValueFloat auf die ID: " . $VarID . " zuwegwiesen werden (" . $S7_Path . "/" . $S7_VAR_ID[0] . ")\r\n";
                    $err_count++;
						 }
				   }
			   else
			      {
				    if (SetValueInteger($VarID,$S7_READ_VALUE) == false)
						 {
					     echo "FEHLER: Wert '" . $S7_READ_VALUE . "' konnte mit SetValueInteger auf die ID: " . $VarID . " zuwegwiesen werden (" . $S7_Path . "/" . $S7_VAR_ID[0] . ")\r\n";
                    $err_count++;
						 }
				   }
			  }
			else
			  {
			  echo ("WARNING: Fehler beim lesen der S7-Variable: " . $S7_VAR_ID[0] . "\r\n");
			  $err_count++;
			  }
    }

   if ($err_count>= 1)
		{
		return $err_count;
		}
	else
		{
		return false;
		}

}




/****************************************************************************
S7_WRITE,
  schreibt einen Wert an die S7 ueber die angegeben S7-Instanz

Paramter:   INSTANZ      Gibt die Instanz an ueber die geschrieben wird
            TYP          (0=bit, 1=byte, 2=word, 3=dword,
				              4=ShortInt, 5=SmallInt, 6=Integer, 7=Real)
            AREA         (0=SystemInfo, 1=SystemFlags,
				              2=AnalogInput CPU200, 3=AnalogOutput CPU200,
				              4=Input, 5=Output, 6=Flags, 7=Datablock,
				              8=InstanzDaten, 9=LokaleDaten, 10=UnknownedArea
				              11=Zaehler, 12=Timer, 13=PEW/PAW)
            AREAADDRESS  (Baustein)
            ADDRES       (StartByte)
            BIT          (Bit = wird nur bei TYP=0 verarbeitet)

				VALUE        Wert der geschrieben werden soll

Example:    S7_WRITE(123456,0,6,0,0,1,true);  //M0.1=1 setzen


ReleaseNotes:
02.06.2012 Gusenbauer
 + Function erstellt und geprft
10.06.2012 Gusenbauer
 + Check der Instanzen ob OKAY,
	sonst Abbruch und RET = FALSE
23.12.2012 Gusenbauer
 + Anpassungen IPS2.7
01.09.2013 Gusenbauer
 + Anpassungen IPS3.0
	==> Instanzenflag "ReadOnly" und "EmulateStatus" gesetzt
 + alten Kompatiblittscode entfernt

*****************************************************************************/

function S7_WRITE($S7_Instanz_ID, $S7_Typ, $S7_Area, $S7_AreaAddress, $S7_Address, $S7_Bit, $S7_WriteValue)
{
	$ret = false;

	//ParentInstanz umkonfigurieren
   IPS_SetProperty($S7_Instanz_ID,"DataType",$S7_Typ);
   IPS_SetProperty($S7_Instanz_ID,"Area",$S7_Area);
   IPS_SetProperty($S7_Instanz_ID,"AreaAddress",$S7_AreaAddress);
   IPS_SetProperty($S7_Instanz_ID,"Address",$S7_Address);
   IPS_SetProperty($S7_Instanz_ID,"Bit",$S7_Bit);

   //Polltimer sicherheitshalber ausschalten
   IPS_SetProperty($S7_Instanz_ID,"Poller",0);

   //ReadOnly an der Instanz entfernen
   IPS_SetProperty($S7_Instanz_ID,"ReadOnly",false);

   //EmulateStatus an der Instanz entfernen
	IPS_SetProperty($S7_Instanz_ID,"EmulateStatus",false);

   //Instanzkonfiguration speichern und Schreibauftrag ausloesen
	if (IPS_ApplyChanges($S7_Instanz_ID) == true)
	   {

	    //Check ob S7-Instanz okay ???
	    $instance = IPS_GetInstance($S7_Instanz_ID);
       if ($instance['InstanceStatus'] >= 104)
          {
           IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Instanzenfehler aufgetreten !! --> ABBRUCH !!");
		     return false;
          }

	    //Check ob Verbindungsinstanz zur Steuerung wirklich steht ???
	    $instance = IPS_GetInstance($instance['ConnectionID']);
       if ($instance['InstanceStatus'] >= 104)
          {
           IPS_LogMessage("S7_FUNCTION",
		                 "ERROR: S7-Verbindungsinstanzenfehler aufgetreten !! --> ABBRUCH !!");
		     return false;
          }


		 if ($S7_Typ == 0)
			 {
	        $ret = S7_WriteBit($S7_Instanz_ID, ConvertToBoolean($S7_WriteValue));
			 }
		 else if ($S7_Typ == 1)
			 {
	        $ret = S7_WriteByte($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 2)
			 {
	        $ret = S7_WriteWord($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 3)
			 {
	        $ret = S7_WriteDWord($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 4)
			 {
	        $ret = S7_WriteShortInt($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 5)
			 {
	        $ret = S7_WriteSmallInt($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 6)
			 {
	        $ret = S7_WriteInteger($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else if ($S7_Typ == 7)
			 {
	        $ret = S7_WriteReal($S7_Instanz_ID, $S7_WriteValue);
			 }
		 else
			 {
			  $ret = false;
			 }


		 if ($ret == false)
			 {
		     IPS_LogMessage("S7_WRITE", "Fehler Schreiben der Instanz: "
			                             . $S7_Instanz_ID . " ==> ("
			                             . $S7_Typ . ","
												  . $S7_Area . ","
												  . $S7_AreaAddress . ","
												  . $S7_Address . ","
												  . $S7_Bit . ") = "
												  . $S7_WriteValue
                                      );
			 }

      }
	else
		{
		 IPS_LogMessage("S7_WRITE", "Fehler beim umkonfigurieren der Instanz : " . $S7_Instanz_ID);
		 $ret = false;
		}


	return $ret;
}

//****************************************************************************


/****************************************************************************
CreateChange_S7_Instances,
  legt bzw. aendert S7-Instanzen

Example:    //index0    = INSTANZNAME DER S7-INSTANZ
            //index1.0  = TYP                  (0=bit, 1=byte, 2=word, 3=dword,
				//                                  4=ShortInt, 5=SmallInt, 6=Integer, 7=Real)
            //index1.1  = AREA                 (0=SystemInfo, 1=SystemFlags,
				//                                  2=AnalogInput CPU200, 3=AnalogOutput CPU200,
				//                                  4=Input, 5=Output, 6=Flags, 7=Datablock,
				//                                  8=InstanzDaten, 9=LokaleDaten, 10=UnknownedArea
				//                                  11=Zaehler, 12=Timer, 13=PEW/PAW)
            //index1.2  = AREAADDRESS          (Baustein)
            //index1.3  = ADDRES               (StartByte)
            //index1.4  = BIT                  (Bit = wird nur bei TYP=0 verarbeitet)
            //index1.5  = POOLER               (PoolInterval in ms)
            //index1.6  = READONLY             (ReadOnlyFlag)
            //index1.7  = EMULATESTATUS        (EmulationsStatus)
            //index1.8  = STATUSVARIABLENNAME  (Name der Statusvariable wenn ungleich "")
            //index1.9  = USERVARIABLENPROFIL  (Uservariablenprofil wenn ungleich "")
            //index1.10 = ACTIONSCRIPT         (Actionscript wenn ungleich 0)

            $S7_INSTANCE_IDs = array("M0.1"            => array(0,6, 0, 0,1,    0,0,1, ""    ,"B_ICON_NOTAUS" ,26768),
                                     "TEST_DB"         => array(2,7,10,26,0,60000,0,1, "TEST",""              ,0    ));
            CreateChange_S7_Instances(123456,$S7_VAR_IDs);

ReleaseNotes:
13.05.2014 Gusenbauer
 + Function erstellt und geprft (#101)
14.05.2014 Gusenbauer
 + Function um Userprofil und Actionscript erweitert (#102)
 
****************************************************************************/
function CreateChange_S7_Instances($S7_ParentInstanz_ID,$S7_INSTANCE_IDs)
{
  while($S7_VAR_ID = each($S7_INSTANCE_IDs))
    {
      $Check_ID = @IPS_GetInstanceIDByName($S7_VAR_ID[0],IPS_GetObject($_IPS['SELF'])['ParentID']);

       if ( ($Check_ID) == 0 ||
            (IPS_GetInstance($Check_ID)['ModuleInfo']['ModuleID'] != "{932076B1-B18E-4AB6-AB6D-275ED30B62DB}")
          )
          {
           //INSTANZ mit diesen Namen existiert nicht
           $S7_Instanz_ID = IPS_CreateInstance("{932076B1-B18E-4AB6-AB6D-275ED30B62DB}");

           //Instanz auf Scriptebene anlegen
           IPS_SetParent($S7_Instanz_ID,IPS_GetObject($_IPS['SELF'])['ParentID']);

           //Instanzname aendern
           IPS_SetName($S7_Instanz_ID,$S7_VAR_ID[0]);

           IPS_LogMessage($_IPS['SELF'], "Generiere neue S7-Instance '" . $S7_Instanz_ID . " - " . $S7_VAR_ID[0] . "'");
          }
       else
          {
           $S7_Instanz_ID = $Check_ID;
          }

       if (IPS_IsInstanceCompatible($S7_Instanz_ID,$S7_ParentInstanz_ID) == true)
          {
           IPS_DisconnectInstance($S7_Instanz_ID);
           IPS_ConnectInstance($S7_Instanz_ID,$S7_ParentInstanz_ID);
           IPS_LogMessage($_IPS['SELF'], "Verbindungsinstance zur S7-Instance '" . $S7_Instanz_ID . " ==> " . $S7_ParentInstanz_ID . "'");
          }


      IPS_LogMessage($_IPS['SELF'], "S7-Instance '" . $S7_Instanz_ID . " - " . $S7_VAR_ID[0] . "' konfigurieren");

      //Instanz konfigurieren
      IPS_SetProperty($S7_Instanz_ID,"DataType",$S7_VAR_ID[1][0]);
      IPS_SetProperty($S7_Instanz_ID,"Area",$S7_VAR_ID[1][1]);
      IPS_SetProperty($S7_Instanz_ID,"AreaAddress",$S7_VAR_ID[1][2]);
      IPS_SetProperty($S7_Instanz_ID,"Address",$S7_VAR_ID[1][3]);
      IPS_SetProperty($S7_Instanz_ID,"Bit",$S7_VAR_ID[1][4]);
      IPS_SetProperty($S7_Instanz_ID,"Poller",$S7_VAR_ID[1][5]);
      IPS_SetProperty($S7_Instanz_ID,"ReadOnly",$S7_VAR_ID[1][6]);
      IPS_SetProperty($S7_Instanz_ID,"EmulateStatus",$S7_VAR_ID[1][7]);
      IPS_ApplyChanges($S7_Instanz_ID);

      if ($S7_VAR_ID[1][8] != "")
         {
          IPS_SetName(IPS_GetObjectIDByIdent("Value",$S7_Instanz_ID), $S7_VAR_ID[1][8]);
         }
      else
         {
          //aendern auf ORIGNALNAME
          IPS_SetName(IPS_GetObjectIDByIdent("Value",$S7_Instanz_ID),"Value");
         }
         
      if ($S7_VAR_ID[1][9] != "")
         {
          IPS_SetVariableCustomProfile(IPS_GetObjectIDByIdent("Value",$S7_Instanz_ID), $S7_VAR_ID[1][9]);
         }
      else
         {
          //Variablenprofil loeschen
          IPS_SetVariableCustomProfile(IPS_GetObjectIDByIdent("Value",$S7_Instanz_ID), "");
         }

      if ($S7_VAR_ID[1][10] >= 1)
         {
          IPS_SetVariableCustomAction(IPS_GetObjectIDByIdent("Value",$S7_Instanz_ID), $S7_VAR_ID[1][10]);
         }
      else
         {
          //Actionscript loeschen
          IPS_SetVariableCustomAction(IPS_GetObjectIDByIdent("Value",$S7_Instanz_ID), 0);
         }

    }
}
//****************************************************************************

?>
